@@ -1,16 +1,15 @@
 """A device from the Smart Device Management API."""
 
 import logging
-
 # Import traits for registration
-from typing import Callable
+from typing import Awaitable, Callable
 
 from . import camera_traits  # noqa: F401
 from . import device_traits  # noqa: F401
 from . import doorbell_traits  # noqa: F401
 from . import thermostat_traits  # noqa: F401
 from .auth import AbstractAuth
-from .event import AsyncEventCallback, EventMessage, EventProcessingError
+from .event import EventMessage, EventProcessingError
 from .traits import BuildTraits, Command
 
 _LOGGER = logging.getLogger(__name__)
@@ -81,18 +80,29 @@ class Device:
         """Return the raw data string."""
         return self._raw_data
 
-    def add_event_callback(
-        self, event_callback: AsyncEventCallback
+    def add_update_listener(
+        self, target: Callable[[], Awaitable[None]]
     ) -> Callable[[], None]:
-        """Register an AsyncEventCallback for udpates to this device.
+        """Register a simple event listener notified on updates.
 
         The return value is a callable that will unregister the callback.
         """
-        self._callbacks.append(event_callback)
+        async def handle_event(event_message: EventMessage):
+            await target()
+        return self.add_event_callback(handle_event)
+
+    def add_event_callback(
+        self, target: Callable[[EventMessage], Awaitable[None]]
+    ) -> Callable[[], None]:
+        """Register an event callback for updates to this device.
+
+        The return value is a callable that will unregister the callback.
+        """
+        self._callbacks.append(target)
 
         def remove_callback():
             """Remove the event_callback."""
-            self._callbacks.remove(event_callback)
+            self._callbacks.remove(target)
 
         return remove_callback
 
@@ -118,4 +128,4 @@ class Device:
             self._traits[trait_name] = trait
 
         for callback in self._callbacks:
-            await callback.async_handle_event(event_message)
+            await callback(event_message)
