@@ -5,7 +5,7 @@ import json
 import logging
 import re
 from abc import ABC, abstractmethod
-from typing import Optional
+from typing import Awaitable, Callable, Optional
 
 from aiohttp.client_exceptions import ClientError
 from google.api_core.exceptions import GoogleAPIError, NotFound, Unauthenticated
@@ -13,7 +13,7 @@ from google.cloud import pubsub_v1
 
 from .auth import AbstractAuth
 from .device_manager import DeviceManager
-from .event import AsyncEventCallback, EventMessage
+from .event import EventMessage
 from .exceptions import AuthException, ConfigurationException, SubscriberException
 from .google_nest_api import GoogleNestAPI
 
@@ -25,6 +25,14 @@ EXPECTED_SUBSCRIBER_REGEXP = re.compile("projects/.*/subscriptions/.*")
 # Used to catch a topic misconfiguration
 EXPECTED_TOPIC_REGEXP = re.compile("projects/sdm-[a-z]+/topics/.*")
 
+WATCHDOG_CHECK_INTERVAL_SECONDS = 10
+
+# Restart the subscriber after some delay, with exponential backoff
+WATCHDOG_RESTART_DELAY_MIN_SECONDS = 10
+WATCHDOG_RESTART_DELAY_MAX_SECONDS = 300
+# Reset watchdog backoff
+WATCHDOG_RESET_THRESHOLD_SECONDS = 60
+
 
 class AbstractSusbcriberFactory(ABC):
     """Abstract class for creating a subscriber, to facilitate testing."""
@@ -87,7 +95,8 @@ class GoogleNestSubscriber:
         subscriber_id: str,
         subscriber_factory: AbstractSusbcriberFactory = DefaultSubscriberFactory(),
         loop: Optional[asyncio.AbstractEventLoop] = None,
-        watchdog_delay: float = 10,
+        watchdog_check_interval_seconds: float = WATCHDOG_CHECK_INTERVAL_SECONDS,
+        watchdog_restart_delay_min_seconds: float = WATCHDOG_RESTART_DELAY_MIN_SECONDS,
     ):
         """Initialize the subscriber for the specified topic."""
         self._auth = auth
@@ -99,15 +108,17 @@ class GoogleNestSubscriber:
         self._subscriber_future = None
         self._callback = None
         self._healthy = True
-        self._watchdog_delay = watchdog_delay
-        if self._watchdog_delay > 0:
+        self._watchdog_check_interval_seconds = watchdog_check_interval_seconds
+        self._watchdog_restart_delay_min_seconds = watchdog_restart_delay_min_seconds
+        self._watchdog_restart_delay_seconds = watchdog_restart_delay_min_seconds
+        if self._watchdog_check_interval_seconds > 0:
             self._watchdog_task = asyncio.create_task(self._watchdog())
         else:
             self._watchdog_task = None
 
-    def set_update_callback(self, callback: AsyncEventCallback):
+    def set_update_callback(self, target: Callable[[EventMessage], Awaitable[None]]):
         """Register a callback invoked when new messages are received."""
-        self._callback = callback
+        self._callback = target
 
     async def start_async(self):
         """Start the subscriber."""
@@ -142,6 +153,9 @@ class GoogleNestSubscriber:
         if not self._healthy:
             _LOGGER.debug("Subscriber reconnected")
             self._healthy = True
+            self._watchdog_restart_delay_seconds = (
+                self._watchdog_restart_delay_min_seconds
+            )
         if self._subscriber_future:
             self._subscriber_future.add_done_callback(self._done_callback)
 
@@ -150,9 +164,18 @@ class GoogleNestSubscriber:
         _LOGGER.debug("Starting background watchdog thread")
         while True:
             if self._subscriber_future and self._subscriber_future.done():
-                _LOGGER.debug("Watchdog: subscriber shut down; restarting")
+                _LOGGER.debug(
+                    "Watchdog: subscriber shut down; restarting in %s seconds",
+                    self._watchdog_restart_delay_seconds,
+                )
+                await asyncio.sleep(self._watchdog_restart_delay_seconds)
+                self._watchdog_restart_delay_seconds *= 2
+                self._watchdog_restart_delay_seconds = max(
+                    self._watchdog_restart_delay_seconds,
+                    WATCHDOG_RESTART_DELAY_MAX_SECONDS,
+                )
                 await self.start_async()
-            await asyncio.sleep(self._watchdog_delay)
+            await asyncio.sleep(self._watchdog_check_interval_seconds)
 
     def wait(self):
         """Block on the subscriber."""
@@ -207,5 +230,5 @@ class GoogleNestSubscriber:
         if self._device_manager_task.done():
             await self._device_manager_task.result().async_handle_event(event)
         if self._callback:
-            await self._callback.async_handle_event(event)
+            await self._callback(event)
         message.ack()
